@@ -1,7 +1,7 @@
 /*
  * abcm2ps: a program to typeset tunes written in abc format using PostScript
  *
- * Copyright (C) 1998-2008 Jean-Franois Moine
+ * Copyright (C) 1998-2010 Jean-Franois Moine
  *
  * Adapted from abc2ps-1.2.5:
  *  Copyright (C) 1996,1997  Michael Methfessel
@@ -82,11 +82,12 @@ static int def_fmt_done = 0;	/* default format read */
 static struct SYMBOL notitle;
 
 /* memory arena (for clrarena, lvlarena & getarena) */
-#define MAXAREAL 3		/* max area levels:
-				 * 0; global, 1: file, 2: tune generation */
+#define MAXAREAL 2		/* max area levels:
+				 * 0; global, 1: tune */
+#define MAXAREANASZ 8192
 static int str_level;		/* current arena level */
 static struct str_a {
-	char	str[4096];	/* memory area */
+	char	str[MAXAREANASZ]; /* memory area */
 	char	*p;		/* pointer in area */
 	struct str_a *n;	/* next area */
 	int	r;		/* remaining space in area */
@@ -122,9 +123,8 @@ int main(int argc,
 
 	/* initialize */
 	outfn[0] = '\0';
-	clrarena(0);			/* global description */
-	clrarena(1);			/* tune description */
-	clrarena(2);			/* tune generation */
+	clrarena(0);			/* global */
+	clrarena(1);			/* tune */
 	clear_buffer();
 	abc_init((void *(*)(int size)) getarena, /* alloc */
 		0,				/* free */
@@ -152,8 +152,18 @@ int main(int argc,
 		p = *argv;
 		if ((c = *p) == '\0')
 			continue;
-		if (c == '-' && p[1] != '-' && p[strlen(p) - 1] == '-')
-			c = '+';	/* switch off flags with '-x-' */
+		if (c == '-') {
+			if (p[1] == '\0') {		/* '-' alone */
+				if (in_fname != 0) {
+					output_file(sel);
+					sel = 0;
+				}
+				in_fname = "";		/* read from stdin */
+				continue;
+			}
+			if (p[1] != '-' && p[strlen(p) - 1] == '-')
+				c = '+'; /* switch off flags with '-x-' */
+		}
 		if (c == '+') {		/* switch off flags with '+' */
 			while (*++p != '\0') {
 				switch (*p) {
@@ -247,14 +257,6 @@ int main(int argc,
 		}
 
 		if (c == '-') {		     /* interpret a flag with '-' */
-			if (p[1] == '\0') {		/* '-' alone */
-				if (in_fname != 0) {
-					output_file(sel);
-					sel = 0;
-				}
-				in_fname = "";		/* read from stdin */
-				continue;
-			}
 			if (p[1] == '-') {		/* long argument */
 				p += 2;
 				if (--argc <= 0) {
@@ -274,7 +276,10 @@ int main(int argc,
 					cfmt.continueall = 1;
 					lock_fmt(&cfmt.continueall);
 					break;
-				case 'E': epsf = 1; break;
+				case 'E':
+					close_output_file();
+					epsf = 1;
+					break;
 				case 'f':
 					cfmt.flatbeams = 1;
 					lock_fmt(&cfmt.flatbeams);
@@ -464,6 +469,11 @@ int main(int argc,
 						}
 						break;
 					case 'O':
+						if (strlen(aaa) >= sizeof outfn) {
+							fprintf(stderr,
+								"++++ '-O' too large\n");
+							exit(1);
+						}
 						strcpy(outfn, aaa);
 						break;
 					case 's':
@@ -548,8 +558,6 @@ static void output_file(char *sel)
 	if (!epsf)
 		open_output_file();
 	fprintf(stderr, "File %s\n", in_fname);
-	clrarena(1);
-	lvlarena(0);
 	t = abc_parse(file);
 	free(file);
 
@@ -605,9 +613,7 @@ static void do_select(struct abctune *t,
 		print_tune = i >= first_tune && i <= last_tune;
 		if (print_tune
 		    || !t->client_data) {	/* (parse the global symbols) */
-			lvlarena(2);
 			do_tune(t, !print_tune);
-			clrarena(2);
 			t->client_data = (void *) 1;	/* treated */
 		}
 		t = t->next;
@@ -740,7 +746,7 @@ static void usage(void)
 		"             if 'b', display in a box\n"
 		"     -k n[b] same as '-j' (abc2ps compatibility)\n"
 		"     -b n    set the first measure number to n\n"
-		"     -f      have flat beams when bagpipe tunes\n"
+		"     -f      have flat beams\n"
 		"     -T n[v]   output the tablature 'n' for voice 'v' / all voices\n"
 		"  .line breaks:\n"
 		"     -c      auto line break\n"
@@ -863,7 +869,13 @@ char *getarena(int len)
 
 	a_p = str_c[str_level];
 	len = (len + 7) & ~7;		/* align at 64 bits boundary */
-	if (a_p->r < len) {
+	if (len > a_p->r) {
+		if (len > MAXAREANASZ) {
+			fprintf(stderr,
+				"++++ getarena - data too wide %d - aborting\n",
+				len);
+			exit(1);
+		}
 		if (a_p->n == 0) {
 			a_p->n = malloc(sizeof *str_r[0]);
 			a_p->n->n = 0;
