@@ -1,7 +1,7 @@
 /*++
  * Generic ABC parser.
  *
- * Copyright (C) 1998-2008 Jean-Franois Moine
+ * Copyright (C) 1998-2009 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996, 1997  Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -139,8 +139,7 @@ static char *parse_basic_note(char *p,
 static void parse_info(struct abctune *t,
 		       char *p,
 		       char *comment);
-static char *parse_gchord(char *p,
-			  int *p_more_gch);
+static char *parse_gchord(char *p);
 static int parse_line(struct abctune *t,
 		      char *p);
 static char *parse_note(struct abctune *t,
@@ -360,6 +359,7 @@ struct abctune *abc_parse(char *file_api)
 			last_tune = t;
 			ulen = gulen;
 			p_micro = t->micro_tb;
+			meter = 0;
 		}
 
 		/* parse the music line */
@@ -606,10 +606,18 @@ static void parse_clef(struct abcsym *s,
 	int clef = -1;
 	int transpose = 0;
 	int clef_line = 2;
+	char str[80];
 
+	str[0] = '\0';
 	if (name != 0 && strncmp(name, "clef=", 5) == 0) {
 		name += 5;
 		switch (*name) {
+		case '\"':
+			name = get_str(str, name, sizeof str);
+			s->u.clef.name = alloc_f(strlen(str) + 1);
+			strcpy(s->u.clef.name, str);
+			clef = TREBLE;
+			break;
 		case 'g':
 			transpose = -7;
 		case 'G':
@@ -699,20 +707,24 @@ static void parse_clef(struct abcsym *s,
 		/* 'middle=<note pitch>' */
 		curvoice->add_pitch = 0;
 		parse_basic_note(middle, &pit, &len, &acc, &nostem);
-		l = 20;
 		if (clef < 0)
 			clef = TREBLE;
 		switch (clef) {
+		default:
+			l = 20 + 4;
+			break;
 		case ALTO:
-			l = 16;
+			l = 16 + 4;
 			break;
 		case BASS:
-			l = 12;
+			l = 12 + 4;
 			break;
 		}
-		l = l - pit + 4 + 14;
-		clef_line = (l % 7) / 2 + 1;
-		transpose = l / 7 * 7 - 14;
+		clef_line = (l - pit + 14) % 7;
+		if (clef_line & 1)
+			clef_line += 7;
+		clef_line = clef_line / 2 + 1;
+		transpose = l - (clef_line - 1) * 2 - pit;
 		s->u.clef.check_pitch = 0;
 	}
 
@@ -819,8 +831,10 @@ static void parse_key(char *p,
 			break;
 		case 'e':
 		case 'E':
-			if (strncasecmp(p, "exp", 3) == 0)
+			if (strncasecmp(p, "exp", 3) == 0) {
+				s->u.key.exp = 1;
 				break;
+			}
 			goto unk;
 		case 'i':
 		case 'I':
@@ -1049,6 +1063,7 @@ static char *top_err = "Cannot identify meter top";
 	nm = 0;
 	in_parenth = 0;
 	wmeasure = 0;
+	m1 = 0;
 	if (*p == 'N' || *p == 'n')
 		p++;				/* no meter */
 	else while (*p != '\0') {
@@ -1132,6 +1147,7 @@ static char *top_err = "Cannot identify meter top";
 		else if (*p == '+')
 			s->u.meter.meter[nm++].top[0] = *p++;
 	}
+	meter = m1;
 	if (*p == '=') {
 		if (sscanf(++p, "%d/%d", &m1, &m2) != 2
 		    || m1 <= 0
@@ -1476,7 +1492,8 @@ static struct kw_s {
 	{"dyn=", 4, 8},
 	{"lyrics=", 7, 9},
 	{"scale=", 6, 10},
-	{0, 0, 0}
+	{"gchord=", 7, 11},
+	{0}
 };
 	struct kw_s *kw;
 
@@ -1576,6 +1593,9 @@ static struct kw_s {
 				p++;
 			break;
 		    }
+		case 11:		/* gchord= */
+			p_stem = &s->u.voice.gchord;
+			break;
 		}
 	}
 	if (clef_name != 0 || clef_middle != 0 || clef_lines != 0 || clef_scale != 0) {
@@ -1620,10 +1640,15 @@ static char *parse_bar(struct abctune *t,
 		break;
 	}
 	p--;
-	if ((bar_type & 0x0f) == B_OBRA && bar_type != B_OBRA) {
-		bar_type >>= 4;		/* have an other bar for '[' */
+
+	/* if the last element is '[', it may start
+	 * a chord, an embedded header or an other bar */
+	if ((bar_type & 0x0f) == B_OBRA && bar_type != B_OBRA
+	    && *p != ' ') {
+		bar_type >>= 4;
 		p--;
 	}
+
 	if (bar_type == (B_OBRA << 8) + (B_BAR << 4) + B_CBRA)	/* [|] */
 		bar_type = (B_OBRA << 4) + B_CBRA;		/* [] */
 
@@ -1863,15 +1888,9 @@ static char *parse_decoline(char *p)
 			syntax("'\\' ignored", p);
 			p++;
 			continue;
-		case '"': {
-			int more_gch;
-
-			p = parse_gchord(p + 1, &more_gch);
-			if (more_gch)
-				syntax("No end of guitar chord / annotation in deco line",
-				       p);
+		case '"':
+			p = parse_gchord(p + 1);
 			break;
-		    }
 		case '!':
 		case '+':
 			p = get_deco(p + 1, &d);
@@ -1920,19 +1939,17 @@ static char *parse_decoline(char *p)
 }
 
 /* -- parse a guitar chord / annotation -- */
-static char *parse_gchord(char *p,
-			  int *p_more_gch)
+static char *parse_gchord(char *p)
 {
 	char *q;
-	int l, l2, more_gch;
+	int l, l2;
 
-	more_gch = 0;
 	q = p;
 	while (*p != '"') {
 		if (*p == '\\')
 			p++;
 		if (*p == '\0') {
-			more_gch = 1;
+			syntax("No end of guitar chord", p);
 			break;
 		}
 		p++;
@@ -1941,11 +1958,11 @@ static char *parse_gchord(char *p,
 	if (gchord) {
 		char *gch;
 
-		/* many guitar chords: concatenate with '\n' or ';' */
+		/* many guitar chords: concatenate with '\n' */
 		l2 = strlen(gchord);
 		gch = alloc_f(l2 + 1 + l + 1);
 		strcpy(gch, gchord);
-		gch[l2++] = more_gch ? ';' : '\n';
+		gch[l2++] = '\n';
 		strncpy(&gch[l2], q, l);
 		gch[l2 + l] = '\0';
 		if (free_f)
@@ -1958,7 +1975,6 @@ static char *parse_gchord(char *p,
 	}
 	if (*p != '\0')
 		p++;
-	*p_more_gch = more_gch;
 	return p;
 }
 
@@ -2149,23 +2165,11 @@ again:					/* for history */
 	while (*p != '\0') {
 		colnum = p - scratch_line;
 		switch (char_tb[(unsigned) *p++]) {
-		case CHAR_GCHORD: {			/* " */
-			int more_gch;
-
+		case CHAR_GCHORD:			/* " */
 			if (flags & ABC_F_GRACE)
 				goto bad_char;
-			for (;;) {
-				p = parse_gchord(p, &more_gch);
-				if (!more_gch)
-					break;
-				if ((p = get_line()) == 0) {
-					syntax("EOF reached while parsing guitar chord",
-					       p);
-					return 1;
-				}
-			}
+			p = parse_gchord(p);
 			break;
-		    }
 		case CHAR_GR_ST:		/* '{' */
 			if (flags & ABC_F_GRACE)
 				goto bad_char;
@@ -2197,21 +2201,13 @@ again:					/* for history */
 				break;
 			}
 			if (p[-1] == '.') {
-				if (*p == '(') {
-					flags |= ABC_F_DOTTED_SLUR;
+				if (*p == '(' || *p == '-' || *p == ')')
 					break;
-				}
-				if (*p == '-') {
-					flags |= ABC_F_DOTTED_TIE;
-					break;
-				}
 				if (*p == '|') {
 					p = parse_bar(t, p + 1);
 					t->last_sym->u.bar.dotted = 1;
 					break;
 				}
-				if (*p == ')')
-					break;
 			}
 			p = parse_deco(p - 1, &dc);
 			dc.h = dc.s = dc.n;
@@ -2248,8 +2244,9 @@ again:					/* for history */
 			syntax("'\\' ignored", p - 1);
 			break;
 		case CHAR_OBRA:			/* '[' */
-			if (*p == '|' || *p == ']'
-			    || isdigit((unsigned) *p) || *p == '"') {
+			if (*p == '|' || *p == ']' || *p == ':'
+			    || isdigit((unsigned) *p) || *p == '"'
+			    || *p == ' ') {
 				if (flags & ABC_F_GRACE)
 					goto bad_char;
 				p = parse_bar(t, p);
@@ -2265,8 +2262,11 @@ again:					/* for history */
 			}
 
 			/* embedded header */
+#if 0
+/*fixme:OK for [I:staff n], ?? for other headers*/
 			if (flags & ABC_F_GRACE)
 				goto bad_char;
+#endif
 			c = ']';
 			q = p;
 			while (*p != '\0' && *p != c)
@@ -2346,7 +2346,9 @@ again:					/* for history */
 				vover = -1;		/* multi-bars */
 				break;
 			}
-			slur <<= 2;
+			slur <<= 3;
+			if (p[-2] == '.')
+				slur |= SL_DOTTED;
 			switch (*p) {
 			case '\'':
 				slur += SL_ABOVE;
@@ -2407,27 +2409,27 @@ again:					/* for history */
 			if (curvoice->last_note == 0
 			    || curvoice->last_note->type != ABC_T_NOTE)
 				goto bad_char;
+			if (p[-2] == '.')
+				tie_pos = SL_DOTTED;
+			else
+				tie_pos = 0;
 			switch (*p) {
 			case '\'':
-				tie_pos = SL_ABOVE;
+				tie_pos += SL_ABOVE;
 				p++;
 				break;
 			case ',':
-				tie_pos = SL_BELOW;
+				tie_pos += SL_BELOW;
 				p++;
 				break;
 			default:
-				tie_pos = SL_AUTO;
+				tie_pos += SL_AUTO;
 				break;
 			}
 			for (i = 0; i <= curvoice->last_note->u.note.nhd; i++) {
 				if (curvoice->last_note->u.note.ti1[i] == 0)
 					curvoice->last_note->u.note.ti1[i] = tie_pos;
 			}
-			if (flags & ABC_F_DOTTED_TIE) {
-				curvoice->last_note->flags |= ABC_F_DOTTED_TIE;
-				flags &= ~ABC_F_DOTTED_TIE;
-			}
 			break;
 		    }
 		case CHAR_BRHY:			/* '>' and '<' */
@@ -2554,12 +2556,16 @@ static char *parse_note(struct abctune *t,
 		int tmp;
 
 		if (chord) {
-			if (*p == '.' && p[1] == '(') {
-				s->flags |= ABC_F_DOTTED_SLUR;
-				p++;
+			if (m >= MAXHD) {
+				syntax("Too many notes in chord", p);
+				m--;
 			}
+			if (*p == '.' && p[1] == '(')
+				p++;
 			if (*p == '(') {
-				s->u.note.sl1[m] <<= 2;
+				s->u.note.sl1[m] <<= 3;
+				if (p[-1] == '.')
+					s->u.note.sl1[m] |= SL_DOTTED;
 				p++;
 				switch (*p) {
 				case '\'':
@@ -2606,11 +2612,8 @@ static char *parse_note(struct abctune *t,
 			if (chord) {
 				for (;;) {
 					if (*p == '.') {
-						if (p[1] == '-')
-							s->flags |= ABC_F_DOTTED_TIE;
-						else if (p[1] == '-')
-							;
-						else	break;
+						if (p[1] != '-')
+							break;
 						p++;
 					}
 					if (*p == '-') {
@@ -2629,7 +2632,8 @@ static char *parse_note(struct abctune *t,
 						}
 					} else if (*p == ')')
 						s->u.note.sl2[m]++;
-					else	break;
+					else
+						break;
 					p++;
 				}
 			}
