@@ -1793,46 +1793,16 @@ void draw_measnb(void)
 {
 	struct SYMBOL *s;
 	char *showm;
-	int bar_time, any_nb, wmeasure;
+	int any_nb, bar_num;
 	float x, y, w;
 
 	showm = cfmt.measurebox ? "showb" : "show";
 	any_nb = 0;
 
-	/* get the current bar number */
-/*fixme: what to do if no symbol in the 1st voice?*/
-	if ((s = voice_tb[cursys->top_voice].sym) == 0
-	    || (s = s->next) == 0)
-		return;
-	for ( ; s->next != 0; s = s->next) {
-		switch (s->type) {
-		case TIMESIG:
-		case CLEF:
-		case KEYSIG:
-		case FMTCHG:
-		case STBRK:
-			continue;
-		case BAR:
-			if (s->u != 0)
-				nbar = s->u;		/* (%%setbarnb) */
-			else if (s->as.u.bar.repeat_bar
-				 && s->as.text != 0
-				 && cfmt.contbarnb == 0) {
-				if (s->as.text[0] == '1')
-					nbar_rep = nbar;
-				else	nbar = nbar_rep; /* restart bar numbering */
-			}
-			break;
-		default:
-			break;
-		}
-		break;
-	}
-	if (nbar > 1) {
-		if (s->prev->type != CLEF)
-			s = s->prev;
+	s = tsfirst;				/* clef */
+	bar_num = nbar;
+	if (bar_num > 1) {
 		if (cfmt.measurenb == 0) {
-			s = s->prev;		/* clef */
 			set_font(MEASUREFONT);
 			any_nb = 1;
 			x = 0;
@@ -1842,15 +1812,28 @@ void draw_measnb(void)
 				y = staff_tb[0].topbar + 14;
 			PUT0("0 ");
 			puty(y);
-			PUT2("y0 M(%d)%s",nbar, showm);
+			PUT2("y0 M(%d)%s",bar_num, showm);
 			y_set(s, 1, x, w, y + cfmt.font_tb[MEASUREFONT].size + 2);
-		} else if (nbar % cfmt.measurenb == 0) {
+		} else if (bar_num % cfmt.measurenb == 0) {
+			for ( ; ; s = s->ts_next) {
+				switch (s->type) {
+				case TIMESIG:
+				case CLEF:
+				case KEYSIG:
+				case FMTCHG:
+				case STBRK:
+					continue;
+				}
+				break;
+			}
+			if (s->prev->type != CLEF)
+				s = s->prev;
 			x = s->x - s->wl;
 			set_font(MEASUREFONT);
 			any_nb = 1;
 			w = cwid('0') * cfmt.font_tb[MEASUREFONT].size;
-			if (nbar >= 10) {
-				if (nbar >= 100)
+			if (bar_num >= 10) {
+				if (bar_num >= 100)
 					w *= 3;
 				else	w *= 2;
 			}
@@ -1861,58 +1844,29 @@ void draw_measnb(void)
 				y = staff_tb[0].topbar + 6;
 			y += 2;
 			putxy(x, y);
-			PUT2("y0 M(%d)%s", nbar, showm);
+			PUT2("y0 M(%d)%s", bar_num, showm);
 			y += cfmt.font_tb[MEASUREFONT].size;
 			y_set(s, 1, x, w, y);
 			s->ymx = y;
 		}
 	}
 
-/*fixme: KO when no bar at the end of the previous line */
-	wmeasure = voice_tb[cursys->top_voice].meter.wmeasure;
-	bar_time = voice_tb[cursys->top_voice].sym->time + wmeasure;
-	for (s = voice_tb[cursys->top_voice].sym->next; s != 0; s = s->next) {
-		switch (s->type) {
-		case TIMESIG:
-			wmeasure = s->as.u.meter.wmeasure;
-			bar_time = s->time + wmeasure;
+	for ( ; s != 0; s = s->ts_next) {
+		if (s->type != BAR
+		 || s->u == 0)
 			continue;
-		case MREST:
-			nbar += s->as.u.bar.len - 1;
-			continue;
-		default:
-			continue;
-		case BAR:
-			break;
-		}
-		if (s->u != 0)
-			nbar = s->u;		/* (%%setbarnb) */
-		if (s->time < bar_time)		/* incomplete measure */
-			continue;
-		if (s->u == 0) {
-			nbar++;
-			if (s->as.u.bar.repeat_bar
-			    && s->as.text != 0
-			    && cfmt.contbarnb == 0) {
-				if (s->as.text[0] == '1')
-					nbar_rep = nbar;
-				else	nbar = nbar_rep; /* restart bar numbering */
-			}
-		}
-		bar_time = s->time + wmeasure;
-		if (s->as.u.bar.repeat_bar
-		    || s->next == 0
-		    || cfmt.measurenb == 0
-		    || (nbar % cfmt.measurenb) != 0
-		    || nbar <= 1)
+		bar_num = s->u;
+		if (cfmt.measurenb == 0
+		 || (bar_num % cfmt.measurenb) != 0
+		 || s->next == 0)
 			continue;
 		if (!any_nb) {
 			any_nb = 1;
 			set_font(MEASUREFONT);
 		}
 		w = cwid('0') * cfmt.font_tb[MEASUREFONT].size;
-		if (nbar >= 10) {
-			if (nbar >= 100)
+		if (bar_num >= 10) {
+			if (bar_num >= 100)
 				w *= 3;
 			else	w *= 2;
 		}
@@ -1934,13 +1888,14 @@ void draw_measnb(void)
 		y += 2;
 		PUT0(" ");
 		putxy(x, y);
-		PUT2("y0 M(%d)%s",nbar, showm);
+		PUT2("y0 M(%d)%s",bar_num, showm);
 		y += cfmt.font_tb[MEASUREFONT].size;
 		y_set(s, 1, x, w, y);
 		s->ymx = y;
 	}
 	if (any_nb)
 		PUT0("\n");
+	nbar = bar_num;
 }
 
 /* -- get the beat from a time signature -- */
