@@ -314,7 +314,7 @@ void make_font_list(void)
 	used_font[f->font_tb[WORDSFONT].fnum] = 1;
 }
 
-/* -- set the name an information header type -- */
+/* -- set the name of an information header type -- */
 /* the argument is
  *	<letter> [ <possibly quoted string> ]
  * this information is kept in the 'I' information */
@@ -400,7 +400,11 @@ void set_format(void)
 	f->aligncomposer = A_RIGHT;
 	f->notespacingfactor = 1.414;
 	f->stemheight = STEM;
+#ifndef WIN32
 	f->dateformat = strdup("\\%b \\%e, \\%Y \\%H:\\%M");
+#else
+	f->dateformat = strdup("\\%b \\%#d, \\%Y \\%H:\\%M");
+#endif
 	f->gracespace = (65 << 16) | (80 << 8) | 120;	/* left-inside-right - unit 1/10 pt */
 	f->textoption = T_LEFT;
 	f->ndfont = FONT_DYN;
@@ -809,6 +813,8 @@ void interpret_fmt_line(char *w,		/* keyword */
 		if (strcmp(w, "postscript") == 0) {
 			if (!file_initialized)
 				user_ps_add(p);
+			else
+				PUT1("%s\n", p);
 			return;
 		}
 		break;
@@ -957,10 +963,8 @@ void interpret_fmt_line(char *w,		/* keyword */
 	case FORMAT_S: {
 		int l;
 
-		if (*((char **) fd->v) != 0)	/* !!no static allocation!! */
-			free(*((char **) fd->v));
 		l = strlen(p) + 1;
-		*((char **) fd->v) = malloc(l);
+		*((char **) fd->v) = getarena(l);
 		if (*p == '"')
 			get_str(*((char **) fd->v), p, l);
 		else	strcpy(*((char **) fd->v), p);
@@ -1062,6 +1066,8 @@ int read_fmt_file(char *fn)
 	if (strcmp(&line[strlen(line) - 3], ".ps") == 0) {
 		if (!file_initialized)
 			user_ps_add(line);
+		else
+			error(0, 0, "Cannot include the file %s", &line[1]);
 		fclose(fp);
 		return 0;
 	}
@@ -1100,6 +1106,8 @@ int read_fmt_file(char *fn)
 					break;
 				if (!file_initialized)
 					user_ps_add(p);
+				else
+					PUT1("%s\n", p);
 			}
 			continue;
 		}
