@@ -300,7 +300,7 @@ static int may_combine(struct SYMBOL *s)
 static void do_combine(struct SYMBOL *s)
 {
 	struct SYMBOL *s2;
-	int nhd, nhd2;
+	int nhd, nhd2, type;
 
 	nhd = s->nhd;
 	s2 = s->ts_next;
@@ -365,10 +365,14 @@ static void do_combine(struct SYMBOL *s)
 /*fixme:should recalculate yav*/
 		s->ymx = 3 * (s->pits[nhd] - 18) + 4;
 	}
-	if (s->as.u.note.ti1[0] == SL_AUTO)	/* force the tie directions */
-		s->as.u.note.ti1[0] = SL_BELOW;
-	if (s->as.u.note.ti1[nhd] == SL_AUTO)
-		s->as.u.note.ti1[nhd] = SL_ABOVE;
+
+	/* force the tie directions */
+	type = s->as.u.note.ti1[0];
+	if ((type & 0x03) == SL_AUTO)
+		s->as.u.note.ti1[0] = SL_BELOW | (type & ~SL_DOTTED);
+	type = s->as.u.note.ti1[nhd];
+	if ((type & 0x03) == SL_AUTO)
+		s->as.u.note.ti1[nhd] = SL_ABOVE | (type & ~SL_DOTTED);
 
 	delsym(s2);			/* remove the next symbol */
 }
@@ -398,8 +402,11 @@ static void combine_voices(void)
 				if (--i <= 0)
 					break;
 			}
-			if (i > 0)
+			if (i > 0) {
+				while (--r > 0)
+					s = s->next;
 				continue;
+			}
 			r = s->as.u.tuplet.r_plet;
 			for (s2 = s; s2 != 0; s2 = s2->next) {
 				if (s2->type != NOTEREST)
@@ -532,6 +539,7 @@ static void set_clef(int staff)
 				sy = sy->next;	/* keep the starting clef */
 				sy->staff[staff].clef.type = clef_type;
 				sy->staff[staff].clef.line = clef_type == TREBLE ? 2 : 4;
+				last_chg = s;
 			}
 			continue;
 		}
@@ -585,10 +593,14 @@ static void set_clef(int staff)
 			s3 = s;
 		else	s3 = 0;
 #endif
-		time = last_chg == 0 ? 0 : last_chg->time;
+#if 0
+		time = last_chg == 0 ? -1 : last_chg->time;
+#endif
 		for (s2 = s->ts_prev; s2 != last_chg; s2 = s2->ts_prev) {
+#if 0
 			if (s2->time <= time)
 				break;
+#endif
 			if (s2->staff != staff)
 				continue;
 			if (s2->type == BAR) {
@@ -621,8 +633,13 @@ static void set_clef(int staff)
 #if 1 /*fixme:test*/
 			/* have a 2nd choice on beam start */
 #if 1
+#if 1
+			if (s2->sflags & S_BEAM_ST)
+				s3 = s2;
+#else
 			if ((s3->sflags & S_BEAM_ST) == 0)
 				s3 = s2;
+#endif
 #else
 			if ((s2->sflags & S_BEAM_ST)
 			    || (s3->sflags & S_BEAM_ST) == 0)
@@ -643,14 +660,14 @@ static void set_clef(int staff)
 		last_chg = s;
 
 		/* if first change, see if any note before */
-		if (s2 == 0) {
+		if (s2 == 0 || s2->type == STAVES) {
 #if 1 /*fixme:test*/
 			s4 = s3;
 #else
 			if ((s4 = s3) == 0)
 				s4 = s;
 #endif
-			for (s4 = s4->ts_prev; s4 != 0; s4 = s4->ts_prev) {
+			for (s4 = s4->ts_prev; s4 != s2; s4 = s4->ts_prev) {
 				if (s4->staff != staff)
 					continue;
 				if (s4->as.type == ABC_T_NOTE)
@@ -658,7 +675,7 @@ static void set_clef(int staff)
 			}
 
 			/* if no note, change the clef of the staff */
-			if (s4 == 0) {
+			if (s4 == s2) {
 				if (clef_type == TREBLE) {
 					clef_type = BASS;
 					sy->staff[staff].clef.line = 4;
@@ -797,7 +814,10 @@ static float set_graceoffs(struct SYMBOL *s)
 	gspinside = ((cfmt.gracespace >> 8) & 0xff) * 0.1;
 	gspright = (cfmt.gracespace & 0xff) * 0.1;
 	xx = 0;
-	g = s->extra;
+	for (g = s->extra; ; g = g->next) {
+		if (g->type == NOTEREST)
+			break;
+	}
 	g->sflags |= S_BEAM_ST;
 	for ( ; ; g = g->next) {
 		if (g->type != NOTEREST)
@@ -862,11 +882,9 @@ static float gchord_width(struct SYMBOL *s,
 	antype = '\0';
 	sep = '\n';
 	for (;;) {
-		if (sep == '\n') {
-			if (*p != '\0' && strchr("^_<>@", *p) != 0)
-				antype = *p++;
-			else	antype = '\0';
-		}
+		if (*p != '\0' && strchr("^_<>@", *p) != 0)
+			antype = *p++;
+		else	antype = '\0';
 		for (q = p; ; q++) {
 			if (*q == '\\') {
 				q++;
@@ -916,7 +934,7 @@ static float gchord_width(struct SYMBOL *s,
 	}
 /*fixme: pb when "<" only*/
 	s2 = s->prev;
-	if (s2->as.text != 0)
+	if (s2 != 0 && s2->as.text != 0)
 		AT_LEAST(wlw, lspc);
 /*fixme: pb when ">" only*/
 	for (s2 = s->next; s2 != 0; s2 = s2->next) {
@@ -1149,6 +1167,10 @@ static void set_width(struct SYMBOL *s)
 				s->wr += 3.5 * (s->dots - 1);
 		}
 
+		/* if a tremolo on 2 notes, have space for the small beam(s) */
+		if ((s->sflags & S_TREM2) && (s->sflags & S_BEAM_END))
+			AT_LEAST(wlnote, 20);
+
 		wlw = wlnote;
 
 		if (s2 != 0) {
@@ -1545,11 +1567,47 @@ static void set_allsymwidth(void)
 		s2 = s;
 		shrink = space = 0;
 		do {
+			float ymx1, ymn1, ymx2, ymn2;
 
 			/* set the minimum space before and after the symbol */
 			set_width(s2);
 
 			/* calculate the minimum space before the symbol */
+#if 1
+			if (s2->ts_prev == 0)
+				new_val = s2->wl;
+			else
+				new_val = 0;
+			if (s2->type == NOTEREST) {
+				ymx1 = 30;
+				ymn1 = -6;
+			} else {
+				ymx1 = s2->ymx;
+				ymn1 = s2->ymn;
+			}
+			for (s3 = s->ts_prev; s3 != 0; s3 = s3->ts_prev) {
+				if (s3->staff == s2->staff
+				    && (!(s3->as.flags & ABC_F_INVIS)
+					|| s3->voice == s2->voice)
+				    && new_val <= s3->wr + s2->wl) {
+					if (s3->type == NOTEREST) {
+						ymx2 = 30;
+						ymn2 = -6;
+					} else {
+						ymx2 = s3->ymx;
+						ymn2 = s3->ymn;
+					}
+					if (ymn1 <= ymx2
+					     && ymx1 >= ymn2)
+						new_val = s3->wr + s2->wl;
+#if 0
+					else if (s3->wr > s2->wl) {
+						if (new_val < s3->wr)
+							new_val = s3->wr;
+					}
+#endif
+				}
+#else
 /*fixme:use tables of x left and x right indexed by y*/
 			if (s2->type != CLEF || s2->prev == 0)
 				new_val = s2->wl;
@@ -1567,6 +1625,7 @@ static void set_allsymwidth(void)
 							new_val = s3->wr;
 					}
 				}
+#endif
 				if (s3->sflags & S_SEQST)
 					break;
 			}
@@ -1578,8 +1637,14 @@ static void set_allsymwidth(void)
 			if ((s2 = s2->ts_next) == 0)
 				break;
 		} while (!(s2->sflags & S_SEQST));
-		s->shrink = shrink;	/* set the spaces at start of sequence */
-		s->space = space;
+		/* set the spaces at start of sequence */
+		if (shrink == 0 && space == 0) {
+			s->sflags &= ~S_SEQST;	/* no space (clef) */
+			s->time = s->ts_prev->time;
+		} else {
+			s->shrink = shrink;
+			s->space = space;
+		}
 		if ((s = s2) == 0)
 			break;
 	}
@@ -1694,8 +1759,8 @@ static void set_repeat(struct SYMBOL *g,
 	/* second check and replace */
 	i = g->nohdix * n;			/* check if NL later in the line*/
 	for (s2 = s; s2 != 0; s2 = s2->next) {
-		if (s2->sflags & S_NL)
-			goto delrep;
+//		if (s2->sflags & S_NL)
+//			goto delrep;
 		if (s2->type == BAR) {
 			if (--i <= 0)
 				break;
@@ -1709,15 +1774,15 @@ static void set_repeat(struct SYMBOL *g,
 				s2 = s2->prev;
 				break;
 			}
-			if (s2->sflags & S_NL)
-				goto delrep;
+//			if (s2->sflags & S_NL)
+//				goto delrep;
 		}
 	}
 	for (; s2 != 0; s2 = s2->prev) {
 		if (s2->type == BAR || s2->type == CLEF)
 			break;
-		if (s2->sflags & S_NL)
-			goto delrep;
+//		if (s2->sflags & S_NL)
+//			goto delrep;
 	}
 	if (s2 == 0 || s->time - s2->time != dur)
 		goto delrep;	/* the previous measure is not in the music line */
@@ -1737,7 +1802,8 @@ static void set_repeat(struct SYMBOL *g,
 		s3->as.type = ABC_T_REST;
 		s3->dur = s3->as.u.note.lens[0] = dur;
 		s3->as.flags = ABC_F_INVIS;
-		set_width(s3);
+/*fixme: should set many parameters for set_width*/
+//		set_width(s3);
 		if (s3->sflags & S_SEQST)
 			s3->space = set_space(s3);
 		s2->as.u.bar.len = 2;
@@ -1778,8 +1844,10 @@ static void set_repeat(struct SYMBOL *g,
 		s3->type = NOTEREST;
 		s3->as.type = ABC_T_REST;
 		s3->dur = s3->as.u.note.lens[0] = dur;
-		s3->sflags = S_REPEAT | S_BEAM_ST;
-		set_width(s3);
+		s3->sflags &= S_NL | S_SEQST;
+		s3->sflags |= S_REPEAT | S_BEAM_ST;
+/*fixme: should set many parameters for set_width*/
+//		set_width(s3);
 		if (s3->sflags & S_SEQST)
 			s3->space = set_space(s3);
 		if (s2->sflags & S_SEQST)
@@ -1907,8 +1975,8 @@ static struct SYMBOL *set_lines(struct SYMBOL *first,	/* first symbol */
 		nlines = wwidth / lwidth + 0.999;
 		if (nlines <= 1) {
 			if (last != 0)
-				set_nl(last);
-			return 0;
+				last = set_nl(last);
+			return last;
 		}
 		xline = wwidth / nlines;
 		x = indent;
@@ -1956,6 +2024,86 @@ static struct SYMBOL *set_lines(struct SYMBOL *first,	/* first symbol */
 	return s;
 }
 
+/* -- set the bar numbers -- */
+static void set_bar_num(void)
+{
+	struct SYMBOL *s;
+	int bar_time, wmeasure;
+	int bar_num;
+
+	wmeasure = voice_tb[cursys->top_voice].meter.wmeasure;
+	if (wmeasure == 0)				/* if M:none */
+		wmeasure = 1;
+
+	/* don't count a bar at start of line */
+	for (s = tsfirst; ; s = s->ts_next) {
+		if (s == 0)
+			return;
+		switch (s->type) {
+		case TIMESIG:
+		case CLEF:
+		case KEYSIG:
+		case FMTCHG:
+		case STBRK:
+			continue;
+		case BAR:
+			if (s->u != 0) {
+				nbar = s->u;		/* (%%setbarnb) */
+				break;
+			}
+			if (s->as.u.bar.repeat_bar
+			    && s->as.text != 0
+			    && cfmt.contbarnb == 0) {
+				if (s->as.text[0] == '1')
+					nbar_rep = nbar;
+				else {
+					nbar = nbar_rep; /* restart bar numbering */
+					s->u = nbar;
+				}
+			}
+			break;
+		}
+		break;
+	}
+
+	/* set the measure number on the top bars */
+	bar_time = s->time + 1;	/* for incomplete measure at start of tune */
+	bar_num = nbar;
+	for ( ; s != 0; s = s->ts_next) {
+		switch (s->type) {
+		case TIMESIG:
+			wmeasure = s->as.u.meter.wmeasure;
+			bar_time = s->time + wmeasure;
+			break;
+		case MREST:
+			bar_num += s->as.u.bar.len - 1;
+			break;
+		case BAR:
+			if (s->u != 0) {
+				bar_num = s->u;		/* (%%setbarnb) */
+				s->u = 0;
+				if (s->time < bar_time)
+					break;
+			} else {
+				if (s->time < bar_time)	/* incomplete measure */
+					break;
+				bar_num++;
+			}
+			if (s->as.u.bar.repeat_bar
+				 && s->as.text != 0
+				 && cfmt.contbarnb == 0) {
+				if (s->as.text[0] == '1')
+					nbar_rep = bar_num;
+				else		/* restart bar numbering */
+					bar_num = nbar_rep;
+			}
+			s->u = bar_num;
+			bar_time = s->time + wmeasure;
+			break;
+		}
+	}
+}
+
 /* -- cut the tune into music lines -- */
 static void cut_tune(float lwidth, float indent)
 {
@@ -1980,7 +2128,7 @@ static void cut_tune(float lwidth, float indent)
 
 	/* if asked, count the measures and set the EOLNs */
 	if ((i = cfmt.barsperstaff) != 0) {
-		wmeasure = first_voice->meter.wmeasure;
+		wmeasure = voice_tb[cursys->top_voice].meter.wmeasure;
 		bar_time = s->time + wmeasure;
 		s2 = s;
 		for ( ; s != 0; s = s->ts_next) {
@@ -2008,7 +2156,8 @@ static void cut_tune(float lwidth, float indent)
 	xmin = indent;
 	s2 = s;
 	for ( ; s != 0; s = s->ts_next) {
-		if (s->sflags & S_SEQST) {
+		if (!(s->sflags & S_SEQST))
+			continue;
 			xmin += s->shrink;
 			if (xmin > lwidth) {
 				error(0, s, "Line overfull (%.0fpt of %.0fpt)",
@@ -2023,7 +2172,6 @@ static void cut_tune(float lwidth, float indent)
 				xmin = s->shrink;
 				indent = 0;
 			}
-		}
 		if (!(s->sflags & S_EOLN))
 			continue;
 		s2 = set_nl(s);
@@ -2676,7 +2824,7 @@ static void init_music_line(void)
 
 	/* add tempo if any (only one) */
 	if ((s = info['Q' - 'A']) != 0) {
-		p_voice = first_voice;
+		p_voice = &voice_tb[cursys->top_voice];
 		s->type = TEMPO;
 		s->voice = p_voice - voice_tb;
 		s->staff = p_voice->staff;
@@ -2861,7 +3009,7 @@ static void set_global(void)
 					s->prev->head = H_SQUARE;
 				break;
 			case NOTEREST:
-				if (s->sflags & S_TREM)
+				if (s->sflags & S_TREM2)
 					break;
 				if (s->as.flags & ABC_F_SPACE)
 					start_flag = 1;
@@ -3058,6 +3206,17 @@ static void set_overlap(void)
 		    || (s->as.flags & ABC_F_INVIS))
 			continue;
 
+		/* treat the stem on two staves with different directions */
+		if ((s->sflags & S_XSTEM)
+		    && s->ts_prev->stem < 0) {
+			s2 = s->ts_prev;
+			for (m = 0; m <= s2->nhd; m++) {
+				s2->shhd[m] += STEM_XOFF * 2;
+				s2->shac[m] -= STEM_XOFF * 2;
+			}
+			s2->xmx += STEM_XOFF * 2;
+		}
+
 		/* search the next note at the same time on the same staff */
 		s2 = s;
 		for (;;) {
@@ -3355,28 +3514,33 @@ static void set_overlap(void)
 			if (s1->as.u.note.accs[i1] == 0)
 				continue;
 			for (i2 = 0; i2 <= s2->nhd; i2++) {
-				if (s2->as.u.note.accs[i2] != 0) {
-					dp = s1->pits[i1] - s2->pits[i2];
-					if (dp > 5 || dp < -5)
-						continue;
-					if (dp == 0) {
-						s2->as.u.note.accs[i2] = 0;
-						continue;
-					}
-					shft = (dp <= -4 || dp >= 4) ? 4.5 : 7;
-					if (dp > 0) {
-						if (s1->as.u.note.accs[i1] & 0xf8)
-							shft += 2;
-						if (s2->shac[i2] < s1->shac[i1] + shft
-						    && s2->shac[i2] > s1->shac[i1] - shft)
-							s2->shac[i2] = s1->shac[i1] + shft;
-					} else {
-						if (s2->as.u.note.accs[i2] & 0xf8)
-							shft += 2;
-						if (s1->shac[i1] < s2->shac[i2] + shft
-						    && s1->shac[i1] > s2->shac[i2] - shft)
-							s1->shac[i1] = s2->shac[i2] + shft;
+				dp = s1->pits[i1] - s2->pits[i2];
+				if (dp > 5 || dp < -5)
+					continue;
+				if (s2->as.u.note.accs[i2] == 0) {
+					if (s2->shhd[i2] < 0
+					    && dp == 3) {
+						s1->shac[i1] = 9 + 7;
 					}
+					continue;
+				}
+				if (dp == 0) {
+					s2->as.u.note.accs[i2] = 0;
+					continue;
+				}
+				shft = (dp <= -4 || dp >= 4) ? 4.5 : 7;
+				if (dp > 0) {
+					if (s1->as.u.note.accs[i1] & 0xf8)
+						shft += 2;
+					if (s2->shac[i2] < s1->shac[i1] + shft
+					    && s2->shac[i2] > s1->shac[i1] - shft)
+						s2->shac[i2] = s1->shac[i1] + shft;
+				} else {
+					if (s2->as.u.note.accs[i2] & 0xf8)
+						shft += 2;
+					if (s1->shac[i1] < s2->shac[i2] + shft
+					    && s1->shac[i1] > s2->shac[i2] - shft)
+						s1->shac[i1] = s2->shac[i2] + shft;
 				}
 			}
 		}
@@ -3503,6 +3667,8 @@ static void set_stems(void)
 			slen -= 2;
 			ymx = 3 * (s->pits[s->nhd] - 18);
 		} else	ymx = ymn;
+		if (s->u != 0)
+			slen += 2 * s->u;		/* tremolo */
 		if (s->as.flags & ABC_F_STEMLESS) {
 			if (s->stem >= 0) {
 				s->y = ymn;
@@ -3955,7 +4121,7 @@ static void cut_symbols(void)
 	int voice;
 
 	/* set the first symbol of each voice */
-	tsnext->ts_prev = 0;
+	tsfirst->ts_prev = 0;
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
 		p_voice->sym = 0;		/* may have no symbol */
 		voice = p_voice - voice_tb;
@@ -4000,11 +4166,43 @@ static void gen_init(void)
 	}
 }
 
+/* -- update the clefs at start of line -- */
+static void update_clefs(void)
+{
+	struct SYMBOL *s;
+	int staff;
+
+	s = tsfirst;
+	while (s->type == CLEF)
+		s = s->ts_next;
+	for ( ; s != 0; s = s->ts_next) {
+		if (s->type != CLEF)
+			continue;
+		staff = s->staff;
+		cursys->staff[staff].clef.type = s->as.u.clef.type;
+		cursys->staff[staff].clef.line = s->as.u.clef.line;
+		cursys->staff[staff].clef.octave = s->as.u.clef.octave;
+		cursys->staff[staff].clef.invis = s->as.u.clef.invis;
+	}
+}
+
+/* -- show the errors -- */
+static void error_show(void)
+{
+	struct SYMBOL *s;
+
+	for (s = tsfirst; s != 0; s = s->ts_next) {
+		if (s->as.flags & ABC_F_ERROR) {
+			putxy(s->x, staff_tb[s->staff].y + s->y);
+			PUT0("showerror\n");
+		}
+	}
+}
+
 /* -- generate the music -- */
 void output_music(void)
 {
 	struct VOICE_S *p_voice;
-	struct SYMBOL *s;
 	float lwidth, indent;
 
 	/* set the staff system if any STAVES at start of the next line */
@@ -4033,6 +4231,7 @@ void output_music(void)
 		lwidth = 20 CM;
 	}
 	indent = set_indent();
+	set_bar_num();
 	cut_tune(lwidth, indent);
 	alfa_last = 0.1;
 	beta_last = 0;
@@ -4054,20 +4253,15 @@ void output_music(void)
 		draw_all_deco();
 		if (showerror > 1) {
 			showerror = 1;
-			for (s = tsfirst; s != 0; s = s->ts_next) {
-				if (s->as.flags & ABC_F_ERROR) {
-					putxy(s->x, staff_tb[s->staff].y + s->y);
-					PUT0("showerror\n");
-				}
-			}
+			error_show();
 		}
 		bskip(line_height);
 		if (indent != 0)
 			PUT1("%.2f 0 T\n", -indent);
-		if ((tsfirst = tsnext) == 0)
-			break;
+		update_clefs();
+		tsfirst = tsnext;
 		gen_init();
-		if ((tsfirst = tsnext) == 0)
+		if (tsfirst == 0)
 			break;
 		cut_symbols();
 		buffer_eob();
