@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2008 Jean-Franois Moine
+ * Copyright (C) 1998-2009 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -402,6 +402,13 @@ static char ps_head[] =
 	"	0 -6 -5 -6 -5 0 RC\n"
 	"	2 add M 0 -4 RL stroke}!\n"
 
+	/* n x y trem - <n> tremolo on one note */
+	"/trem{	M -4.5 0 RM{\n"
+	"		currentpoint\n"
+	"		9 3 RL 0 -3 RL -9 -3 RL 0 3 RL\n"
+	"		fill 5.4 sub M\n"
+	"	}repeat}!\n"
+
 	/* x y hl - ledger line */
 	"/hl{	.8 SLW M -6 0 RM 12 0 RL stroke}!\n"
 	/* x y hl1 - longer ledger line */
@@ -490,6 +497,12 @@ static char ps_head[] =
 	"	 CharStrings/natchar{pop\n"
 	"		.60 0 0 -.10 .60 .75 setcachedevice\n"
 	"		.056 dup scale 5.8 6 nt0}bind put\n"
+	"	 CharStrings/dsharpchar{pop\n"
+	"		.60 0 0 -.10 .60 .75 setcachedevice\n"
+	"		.056 dup scale 5.8 6 dsh0}bind put\n"
+	"	 CharStrings/dflatchar{pop\n"
+	"		.60 0 0 0 .60 .78 setcachedevice\n"
+	"		.056 dup scale 5.8 5 dft0}bind put\n"
 	"	}{\n"
 	"	 CharStrings/sharpchar{pop\n"
 	"		600 0 0 -100 600 750 setcachedevice\n"
@@ -500,6 +513,12 @@ static char ps_head[] =
 	"	 CharStrings/natchar{pop\n"
 	"		600 0 0 -100 600 750 setcachedevice\n"
 	"		56 dup scale 5.8 6 nt0}bind put\n"
+	"	 CharStrings/dsharpchar{pop\n"
+	"		600 0 0 -100 600 750 setcachedevice\n"
+	"		56 dup scale 5.8 6 dsh0}bind put\n"
+	"	 CharStrings/dflatchar{pop\n"
+	"		600 0 0 0 600 780 setcachedevice\n"
+	"		56 dup scale 5.8 5 dft0}bind put\n"
 	"	 }ifelse\n"
 	/*if RoPS and Font Type 3, change BuildChar*/
 	"	product(RoPS)eq FontType 3 eq and{\n"
@@ -702,6 +721,9 @@ static char ps_head[] =
 	/* x y pfthd - percussion flat head */
 	"/pfthd{/x 2 index def/y 1 index def dsh0\n"
 	"	.7 SLW x y 4 0 360 arc stroke}!\n"
+	/* same for dble sharp/flat */
+	"/pdshhd{pshhd}!\n"
+	"/pdfthd{pfthd}!\n"
 
 	/* x y ghd - grace note head */
 	"/ghd{	xymove\n"
@@ -751,11 +773,11 @@ static char *enc_tb[MAXENC] = {
 	"/Racute/Aacute/Acircumflex/Abreve/Adieresis/Lacute/Cacute/Ccedilla\n"
 	"/Ccaron/Eacute/Eogonek/Edieresis/Ecaron/Iacute/Icircumflex/Dcaron\n"
 	"/Dcroat/Nacute/Ncaron/Oacute/Ocircumflex/Ohungarumlaut/Odieresis/multiply\n"
-	"/Rcaron/Uring/Uacute/Uhungarumlaut/Udieresis/Yacute/Tcedilla/germandbls\n"
+	"/Rcaron/Uring/Uacute/Uhungarumlaut/Udieresis/Yacute/Tcommaaccent/germandbls\n"
 	"/racute/aacute/acircumflex/abreve/adieresis/lacute/cacute/ccedilla\n"
 	"/ccaron/eacute/eogonek/edieresis/ecaron/iacute/icircumflex/dcaron\n"
 	"/dcroat/nacute/ncaron/oacute/ocircumflex/ohungarumlaut/odieresis/divide\n"
-	"/rcaron/uring/uacute/uhungarumlaut/udieresis/yacute/tcedilla/dotaccent",
+	"/rcaron/uring/uacute/uhungarumlaut/udieresis/yacute/tcommaaccent/dotaccent",
 	/* 3 */
 	"/space/Hstroke/breve/sterling/currency/yen/Hcircumflex/section\n"
 	"/dieresis/Idotaccent/Scedilla/Gbreve/Jcircumflex/hyphen/registered/Zdotaccent\n"
@@ -844,7 +866,9 @@ void define_encoding(int enc,		/* index */
 		fprintf(fout,
 			"%s dup 8#201/sharpchar put\n"
 			"dup 8#202/flatchar put\n"
-			"8#203/natchar put\n",
+			"dup 8#203/natchar put\n"
+			"dup 8#204/dsharpchar put\n"
+			"8#205/dflatchar put\n",
 			ename);
 	}
 	fprintf(fout, "/mkfontext%d{\n"
